@@ -2,7 +2,7 @@ import os
 
 HOME_DIR = os.path.join(os.path.dirname(__file__), '..')
 
-from .pytrust import PyTrust
+from .pytrust import PyTrust, help
 from .utils.constants import FeatureTypes, NUMERICAL, CATEGORICAL, REGRESSION, CLASSIFICATION
 from .utils.dmd import DMD
 from .utils.metrics import Metrics
